@@ -1,63 +1,39 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "historycompleter.h"
-#include "fancylineedit.h"
-#include "theme/theme.h"
-#include "utilsicons.h"
 
 #include "qtcassert.h"
-
-#include <QSettings>
+#include "qtcsettings.h"
+#include "utilsicons.h"
 
 #include <QItemDelegate>
-#include <QKeyEvent>
 #include <QListView>
+#include <QMouseEvent>
 #include <QPainter>
 #include <QWindow>
 
 namespace Utils {
 namespace Internal {
 
-static QSettings *theSettings = 0;
+static QtcSettings *theSettings = nullptr;
+const bool isLastItemEmptyDefault = false;
 
 class HistoryCompleterPrivate : public QAbstractListModel
 {
 public:
-    int rowCount(const QModelIndex &parent = QModelIndex()) const;
-    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
-    bool removeRows(int row, int count, const QModelIndex &parent = QModelIndex());
+    int rowCount(const QModelIndex &parent = QModelIndex()) const override;
+    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const override;
+    bool removeRows(int row, int count, const QModelIndex &parent = QModelIndex()) override;
 
     void clearHistory();
     void addEntry(const QString &str);
 
     QStringList list;
-    QString historyKey;
-    QString historyKeyIsLastItemEmpty;
-    int maxLines = 30;
-    bool isLastItemEmpty = false;
+    Key historyKey;
+    Key historyKeyIsLastItemEmpty;
+    int maxLines = 6;
+    bool isLastItemEmpty = isLastItemEmptyDefault;
 };
 
 class HistoryLineDelegate : public QItemDelegate
@@ -69,7 +45,7 @@ public:
         , icon(Icons::EDIT_CLEAR.icon())
     {}
 
-    void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
+    void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const override
     {
         // from QHistoryCompleter
         QStyleOptionViewItem optCopy = option;
@@ -78,11 +54,11 @@ public:
             optCopy.state |= QStyle::State_HasFocus;
         QItemDelegate::paint(painter,option,index);
         // add remove button
-        QWindow *window = view->window()->windowHandle();
-        const QPixmap iconPixmap = icon.pixmap(window, option.rect.size());
+        const qreal devicePixelRatio = painter->device()->devicePixelRatio();
+        const QPixmap iconPixmap = icon.pixmap(option.rect.size(), devicePixelRatio);
         QRect pixmapRect = QStyle::alignedRect(option.direction,
                                                Qt::AlignRight | Qt::AlignVCenter,
-                                               iconPixmap.size() / window->devicePixelRatio(),
+                                               iconPixmap.size() / devicePixelRatio,
                                                option.rect);
         if (!clearIconSize.isValid())
             clearIconSize = pixmapRect.size();
@@ -100,6 +76,10 @@ public:
     HistoryLineView(HistoryCompleterPrivate *model_)
         : model(model_)
     {
+        setEditTriggers(QAbstractItemView::NoEditTriggers);
+        setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+        setSelectionBehavior(QAbstractItemView::SelectRows);
+        setSelectionMode(QAbstractItemView::SingleSelection);
     }
 
     void installDelegate()
@@ -109,7 +89,7 @@ public:
     }
 
 private:
-    void mousePressEvent(QMouseEvent *event)
+    void mousePressEvent(QMouseEvent *event) override
     {
         const QSize clearButtonSize = delegate->clearIconSize;
         if (clearButtonSize.isValid()) {
@@ -117,8 +97,11 @@ private:
             if (layoutDirection() == Qt::LeftToRight)
                 rr = viewport()->width() - event->x();
             if (rr < clearButtonSize.width()) {
-                model->removeRow(indexAt(event->pos()).row());
-                return;
+                const QModelIndex index = indexAt(event->pos());
+                if (index.isValid()) {
+                    model->removeRow(indexAt(event->pos()).row());
+                    return;
+                }
             }
         }
         QListView::mousePressEvent(event);
@@ -154,7 +137,7 @@ bool HistoryCompleterPrivate::removeRows(int row, int count, const QModelIndex &
     beginRemoveRows(parent, row, row + count -1);
     for (int i = 0; i < count; ++i)
         list.removeAt(row);
-    theSettings->setValue(historyKey, list);
+    theSettings->setValueWithDefault(historyKey, list);
     endRemoveRows();
     return true;
 }
@@ -172,7 +155,9 @@ void HistoryCompleterPrivate::addEntry(const QString &str)
     const QString entry = str.trimmed();
     if (entry.isEmpty()) {
         isLastItemEmpty = true;
-        theSettings->setValue(historyKeyIsLastItemEmpty, isLastItemEmpty);
+        theSettings->setValueWithDefault(historyKeyIsLastItemEmpty,
+                                         isLastItemEmpty,
+                                         isLastItemEmptyDefault);
         return;
     }
     int removeIndex = list.indexOf(entry);
@@ -182,23 +167,25 @@ void HistoryCompleterPrivate::addEntry(const QString &str)
     list.prepend(entry);
     list = list.mid(0, maxLines - 1);
     endResetModel();
-    theSettings->setValue(historyKey, list);
+    theSettings->setValueWithDefault(historyKey, list);
     isLastItemEmpty = false;
-    theSettings->setValue(historyKeyIsLastItemEmpty, isLastItemEmpty);
+    theSettings->setValueWithDefault(historyKeyIsLastItemEmpty,
+                                     isLastItemEmpty,
+                                     isLastItemEmptyDefault);
 }
 
-HistoryCompleter::HistoryCompleter(const QString &historyKey, QObject *parent)
+HistoryCompleter::HistoryCompleter(const Key &historyKey, QObject *parent)
     : QCompleter(parent),
       d(new HistoryCompleterPrivate)
 {
     QTC_ASSERT(!historyKey.isEmpty(), return);
     QTC_ASSERT(theSettings, return);
 
-    d->historyKey = QLatin1String("CompleterHistory/") + historyKey;
+    d->historyKey = "CompleterHistory/" + historyKey;
     d->list = theSettings->value(d->historyKey).toStringList();
-    d->historyKeyIsLastItemEmpty = QLatin1String("CompleterHistory/")
-        + historyKey + QLatin1String(".IsLastItemEmpty");
-    d->isLastItemEmpty = theSettings->value(d->historyKeyIsLastItemEmpty, false).toBool();
+    d->historyKeyIsLastItemEmpty = "CompleterHistory/" + historyKey + ".IsLastItemEmpty";
+    d->isLastItemEmpty = theSettings->value(d->historyKeyIsLastItemEmpty, isLastItemEmptyDefault)
+                             .toBool();
 
     setModel(d);
     auto popup = new HistoryLineView(d);
@@ -220,10 +207,10 @@ QString HistoryCompleter::historyItem() const
     return d->list.at(0);
 }
 
-bool HistoryCompleter::historyExistsFor(const QString &historyKey)
+bool HistoryCompleter::historyExistsFor(const Key &historyKey)
 {
     QTC_ASSERT(theSettings, return false);
-    const QString fullKey = QLatin1String("CompleterHistory/") + historyKey;
+    const Key fullKey = "CompleterHistory/" + historyKey;
     return theSettings->value(fullKey).isValid();
 }
 
@@ -257,7 +244,7 @@ void HistoryCompleter::addEntry(const QString &str)
     d->addEntry(str);
 }
 
-void HistoryCompleter::setSettings(QSettings *settings)
+void HistoryCompleter::setSettings(QtcSettings *settings)
 {
     Internal::theSettings = settings;
 }
