@@ -22,6 +22,7 @@
 
 /*!
     \class Utils::ProjectIntroPage
+    \inmodule QtCreator
 
     \brief The ProjectIntroPage class is the standard wizard page for a project,
     letting the user choose its name
@@ -66,7 +67,6 @@ ProjectIntroPage::ProjectIntroPage(QWidget *parent) :
     WizardPage(parent),
     d(new ProjectIntroPagePrivate)
 {
-    resize(355, 289);
     setTitle(Tr::tr("Introduction and Project Location"));
 
     d->m_descriptionLabel = new QLabel(this);
@@ -79,8 +79,10 @@ ProjectIntroPage::ProjectIntroPage(QWidget *parent) :
     d->m_nameLineEdit = new Utils::FancyLineEdit(frame);
 
     d->m_pathChooser = new Utils::PathChooser(frame);
+    d->m_pathChooser->setObjectName("baseFolder"); // used by Squish
     d->m_pathChooser->setExpectedKind(PathChooser::Directory);
     d->m_pathChooser->setDisabled(d->m_forceSubProject);
+    d->m_pathChooser->setAllowPathFromDevice(true);
 
     d->m_projectsDirectoryCheckBox = new QCheckBox(Tr::tr("Use as default project location"));
     d->m_projectsDirectoryCheckBox->setObjectName("projectsDirectoryCheckBox");
@@ -108,7 +110,8 @@ ProjectIntroPage::ProjectIntroPage(QWidget *parent) :
     using namespace Layouting;
 
     Form {
-        Tr::tr("Name:"), d->m_nameLineEdit, br,
+        Tr::tr("Name:"), d->m_nameLineEdit,
+                br,
         d->m_projectLabel, d->m_projectComboBox, br,
         Column { Space(12) }, br,
         Tr::tr("Create in:"), d->m_pathChooser, br,
@@ -133,7 +136,7 @@ ProjectIntroPage::ProjectIntroPage(QWidget *parent) :
     connect(d->m_nameLineEdit, &FancyLineEdit::validReturnPressed,
             this, &ProjectIntroPage::slotActivated);
     connect(d->m_projectComboBox, &QComboBox::currentIndexChanged,
-            this, &ProjectIntroPage::slotChanged);
+            this, &ProjectIntroPage::onCurrentProjectIndexChanged);
 
     setProperty(SHORT_TITLE_PROPERTY, Tr::tr("Location"));
     registerFieldWithName(QLatin1String("Path"), d->m_pathChooser, "path", SIGNAL(textChanged(QString)));
@@ -190,15 +193,10 @@ bool ProjectIntroPage::isComplete() const
 
 bool ProjectIntroPage::validate()
 {
-    if (d->m_forceSubProject) {
-        int index = d->m_projectComboBox->currentIndex();
-        if (index == 0)
-            return false;
-        d->m_pathChooser->setFilePath(d->m_projectDirectories.at(index));
-    }
     // Validate and display status
     if (!d->m_pathChooser->isValid()) {
-        displayStatusMessage(InfoLabel::Error, d->m_pathChooser->errorMessage());
+        if (const QString msg = d->m_pathChooser->errorMessage(); !msg.isEmpty())
+            displayStatusMessage(InfoLabel::Error, msg);
         return false;
     }
 
@@ -257,6 +255,25 @@ void ProjectIntroPage::slotActivated()
         emit activated();
 }
 
+void ProjectIntroPage::onCurrentProjectIndexChanged(int index)
+{
+    if (d->m_forceSubProject) {
+        const int available = d->m_projectDirectories.size();
+        if (available == 0)
+            return;
+        QTC_ASSERT(index < available, return);
+        if (index < 0)
+            return;
+
+        const FilePath current = d->m_projectDirectories.at(index);
+        const FilePath visible = d->m_pathChooser->filePath();
+        if (visible != current && !visible.isChildOf(current))
+            d->m_pathChooser->setFilePath(current);
+
+        fieldsUpdated();
+    }
+}
+
 bool ProjectIntroPage::forceSubProject() const
 {
     return d->m_forceSubProject;
@@ -303,7 +320,7 @@ bool ProjectIntroPage::validateProjectName(const QString &name, QString *errorMe
 
         // if pos is set by validate it is cought at the bottom where it shows
         // a more detailed error message
-        if (validatorState != QValidator::Acceptable && (pos == -1 || pos >= name.count())) {
+        if (validatorState != QValidator::Acceptable && (pos == -1 || pos >= name.size())) {
             if (errorMessage) {
                 if (d->m_projectNameValidatorUserMessage.isEmpty())
                     *errorMessage = Tr::tr("Project name is invalid.");
