@@ -70,9 +70,9 @@ struct ModeManagerPrivate
 
     Internal::FancyTabWidget *m_modeStack;
     Internal::FancyActionBar *m_actionBar;
-    QMap<QAction*, int> m_actions;
-    QVector<IMode*> m_modes;
-    QVector<Command*> m_modeCommands;
+    QHash<QAction *, int> m_actions;
+    QVector<IMode *> m_modes;
+    QVector<Command *> m_modeCommands;
     Context m_addedContexts;
     int m_oldCurrent;
     ModeManager::Style m_modeStyle = ModeManager::Style::IconsAndText;
@@ -311,9 +311,8 @@ void ModeManager::currentTabChanged(int index)
     if (!mode)
         return;
 
-    // FIXME: This hardcoded context update is required for the Debug and Edit modes, since
-    // they use the editor widget, which is already a context widget so the main window won't
-    // go further up the parent tree to find the mode context.
+    // Set the mode's context regardless of focus widget.
+    // Whenever a mode is active, it's Context is active.
     ICore::updateAdditionalContexts(d->m_addedContexts, mode->context());
     d->m_addedContexts = mode->context();
 
@@ -322,6 +321,7 @@ void ModeManager::currentTabChanged(int index)
         oldMode = d->m_modes.at(d->m_oldCurrent);
     d->m_oldCurrent = index;
     emit currentModeChanged(mode->id(), oldMode ? oldMode->id() : Id());
+    emit currentMainWindowChanged();
 }
 
 /*!
