@@ -1,27 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "jsonwizardpagefactory_p.h"
 
@@ -32,20 +10,29 @@
 #include "jsonprojectpage.h"
 #include "jsonsummarypage.h"
 #include "jsonwizardfactory.h"
+#include "jsonwizardpagefactory.h"
+#include "../projectexplorertr.h"
 
 #include <utils/qtcassert.h>
 #include <utils/wizardpage.h>
 
-#include <QCoreApplication>
-#include <QVariant>
+using namespace Utils;
 
-namespace ProjectExplorer {
-namespace Internal {
+namespace ProjectExplorer::Internal {
 
 // --------------------------------------------------------------------
 // FieldPageFactory:
 // --------------------------------------------------------------------
 
+class FieldPageFactory final : public JsonWizardPageFactory
+{
+public:
+    FieldPageFactory();
+
+    WizardPage *create(JsonWizard *wizard, Id typeId, const QVariant &data) final;
+    bool validateData(Id typeId, const QVariant &data, QString *errorMessage) final;
+};
+
 FieldPageFactory::FieldPageFactory()
 {
     setTypeIdsSuffix(QLatin1String("Fields"));
@@ -60,9 +47,9 @@ FieldPageFactory::FieldPageFactory()
     JsonFieldPage::registerFieldFactory(QLatin1String("IconList"), []() { return new IconListField; });
 }
 
-Utils::WizardPage *FieldPageFactory::create(JsonWizard *wizard, Core::Id typeId, const QVariant &data)
+WizardPage *FieldPageFactory::create(JsonWizard *wizard, Id typeId, const QVariant &data)
 {
-    Q_UNUSED(wizard);
+    Q_UNUSED(wizard)
 
     QTC_ASSERT(canCreate(typeId), return nullptr);
 
@@ -76,19 +63,18 @@ Utils::WizardPage *FieldPageFactory::create(JsonWizard *wizard, Core::Id typeId,
     return page;
 }
 
-bool FieldPageFactory::validateData(Core::Id typeId, const QVariant &data, QString *errorMessage)
+bool FieldPageFactory::validateData(Id typeId, const QVariant &data, QString *errorMessage)
 {
     QTC_ASSERT(canCreate(typeId), return false);
 
-    QList<QVariant> list = JsonWizardFactory::objectOrList(data, errorMessage);
+    const QList<QVariant> list = JsonWizardFactory::objectOrList(data, errorMessage);
     if (list.isEmpty()) {
-        *errorMessage = QCoreApplication::translate("ProjectExplorer::JsonWizard",
-                                                    "When parsing fields of page \"%1\": %2")
+        *errorMessage = Tr::tr("When parsing fields of page \"%1\": %2")
                 .arg(typeId.toString()).arg(*errorMessage);
         return false;
     }
 
-    foreach (const QVariant &v, list) {
+    for (const QVariant &v : list) {
         JsonFieldPage::Field *field = JsonFieldPage::Field::parse(v, errorMessage);
         if (!field)
             return false;
@@ -102,26 +88,34 @@ bool FieldPageFactory::validateData(Core::Id typeId, const QVariant &data, QStri
 // FilePageFactory:
 // --------------------------------------------------------------------
 
+class FilePageFactory final : public JsonWizardPageFactory
+{
+public:
+    FilePageFactory();
+
+    WizardPage *create(JsonWizard *wizard, Id typeId, const QVariant &data) override;
+    bool validateData(Id typeId, const QVariant &data, QString *errorMessage) override;
+};
+
 FilePageFactory::FilePageFactory()
 {
     setTypeIdsSuffix(QLatin1String("File"));
 }
 
-Utils::WizardPage *FilePageFactory::create(JsonWizard *wizard, Core::Id typeId, const QVariant &data)
+WizardPage *FilePageFactory::create(JsonWizard *wizard, Id typeId, const QVariant &data)
 {
-    Q_UNUSED(wizard);
-    Q_UNUSED(data);
+    Q_UNUSED(wizard)
+    Q_UNUSED(data)
     QTC_ASSERT(canCreate(typeId), return nullptr);
 
     return new JsonFilePage;
 }
 
-bool FilePageFactory::validateData(Core::Id typeId, const QVariant &data, QString *errorMessage)
+bool FilePageFactory::validateData(Id typeId, const QVariant &data, QString *errorMessage)
 {
     QTC_ASSERT(canCreate(typeId), return false);
-    if (!data.isNull() && (data.type() != QVariant::Map || !data.toMap().isEmpty())) {
-        *errorMessage = QCoreApplication::translate("ProjectExplorer::JsonWizard",
-                                                    "\"data\" for a \"File\" page needs to be unset or an empty object.");
+    if (!data.isNull() && (data.typeId() != QMetaType::QVariantMap || !data.toMap().isEmpty())) {
+        *errorMessage = Tr::tr("\"data\" for a \"File\" page needs to be unset or an empty object.");
         return false;
     }
 
@@ -132,18 +126,27 @@ bool FilePageFactory::validateData(Core::Id typeId, const QVariant &data, QStrin
 // KitsPageFactory:
 // --------------------------------------------------------------------
 
-static const char KEY_PROJECT_FILE[] = "projectFilePath";
-static const char KEY_REQUIRED_FEATURES[] = "requiredFeatures";
-static const char KEY_PREFERRED_FEATURES[] = "preferredFeatures";
+const char KEY_PROJECT_FILE[] = "projectFilePath";
+const char KEY_REQUIRED_FEATURES[] = "requiredFeatures";
+const char KEY_PREFERRED_FEATURES[] = "preferredFeatures";
+
+class KitsPageFactory final : public JsonWizardPageFactory
+{
+public:
+    KitsPageFactory();
+
+    WizardPage *create(JsonWizard *wizard, Id typeId, const QVariant &data) override;
+    bool validateData(Id typeId, const QVariant &data, QString *errorMessage) override;
+};
 
 KitsPageFactory::KitsPageFactory()
 {
     setTypeIdsSuffix(QLatin1String("Kits"));
 }
 
-Utils::WizardPage *KitsPageFactory::create(JsonWizard *wizard, Core::Id typeId, const QVariant &data)
+WizardPage *KitsPageFactory::create(JsonWizard *wizard, Id typeId, const QVariant &data)
 {
-    Q_UNUSED(wizard);
+    Q_UNUSED(wizard)
     QTC_ASSERT(canCreate(typeId), return nullptr);
 
     auto page = new JsonKitsPage;
@@ -160,28 +163,25 @@ static bool validateFeatureList(const QVariantMap &data, const QByteArray &key,
     QString message;
     JsonKitsPage::parseFeatures(data.value(QLatin1String(key)), &message);
     if (!message.isEmpty()) {
-        *errorMessage = QCoreApplication::translate("ProjectExplorer::JsonWizard",
-                                                    "Error parsing \"%1\" in \"Kits\" page: %2")
+        *errorMessage = Tr::tr("Error parsing \"%1\" in \"Kits\" page: %2")
                 .arg(QLatin1String(key), message);
         return false;
     }
     return true;
 }
 
-bool KitsPageFactory::validateData(Core::Id typeId, const QVariant &data, QString *errorMessage)
+bool KitsPageFactory::validateData(Id typeId, const QVariant &data, QString *errorMessage)
 {
     QTC_ASSERT(canCreate(typeId), return false);
 
-    if (data.isNull() || data.type() != QVariant::Map) {
-        *errorMessage = QCoreApplication::translate("ProjectExplorer::JsonWizard",
-                                                    "\"data\" must be a JSON object for \"Kits\" pages.");
+    if (data.isNull() || data.typeId() != QMetaType::QVariantMap) {
+        *errorMessage = Tr::tr("\"data\" must be a JSON object for \"Kits\" pages.");
         return false;
     }
 
     QVariantMap tmp = data.toMap();
     if (tmp.value(QLatin1String(KEY_PROJECT_FILE)).toString().isEmpty()) {
-        *errorMessage = QCoreApplication::translate("ProjectExplorer::JsonWizard",
-                                                    "\"Kits\" page requires a \"%1\" set.")
+        *errorMessage = Tr::tr("\"Kits\" page requires a \"%1\" set.")
                 .arg(QLatin1String(KEY_PROJECT_FILE));
         return false;
     }
@@ -195,16 +195,26 @@ bool KitsPageFactory::validateData(Core::Id typeId, const QVariant &data, QStrin
 // --------------------------------------------------------------------
 
 static const char KEY_PROJECT_NAME_VALIDATOR[] = "projectNameValidator";
+static const char KEY_PROJECT_NAME_VALIDATOR_USER_MESSAGE[] = "trProjectNameValidatorUserMessage";
+
+class ProjectPageFactory final : public JsonWizardPageFactory
+{
+public:
+    ProjectPageFactory();
+
+    WizardPage *create(JsonWizard *wizard, Id typeId, const QVariant &data) override;
+    bool validateData(Id typeId, const QVariant &data, QString *errorMessage) override;
+};
 
 ProjectPageFactory::ProjectPageFactory()
 {
     setTypeIdsSuffix(QLatin1String("Project"));
 }
 
-Utils::WizardPage *ProjectPageFactory::create(JsonWizard *wizard, Core::Id typeId, const QVariant &data)
+WizardPage *ProjectPageFactory::create(JsonWizard *wizard, Id typeId, const QVariant &data)
 {
-    Q_UNUSED(wizard);
-    Q_UNUSED(data);
+    Q_UNUSED(wizard)
+    Q_UNUSED(data)
     QTC_ASSERT(canCreate(typeId), return nullptr);
 
     auto page = new JsonProjectPage;
@@ -215,23 +225,25 @@ Utils::WizardPage *ProjectPageFactory::create(JsonWizard *wizard, Core::Id typeI
     page->setDescription(wizard->expander()->expand(description));
     QString projectNameValidator
             = tmp.value(QLatin1String(KEY_PROJECT_NAME_VALIDATOR)).toString();
+    QString projectNameValidatorUserMessage
+            = JsonWizardFactory::localizedString(tmp.value(QLatin1String(KEY_PROJECT_NAME_VALIDATOR_USER_MESSAGE)));
+
     if (!projectNameValidator.isEmpty()) {
         QRegularExpression regularExpression(projectNameValidator);
         if (regularExpression.isValid())
-            page->setProjectNameRegularExpression(regularExpression);
+            page->setProjectNameRegularExpression(regularExpression, projectNameValidatorUserMessage);
     }
 
     return page;
 }
 
-bool ProjectPageFactory::validateData(Core::Id typeId, const QVariant &data, QString *errorMessage)
+bool ProjectPageFactory::validateData(Id typeId, const QVariant &data, QString *errorMessage)
 {
-    Q_UNUSED(errorMessage);
+    Q_UNUSED(errorMessage)
 
     QTC_ASSERT(canCreate(typeId), return false);
-    if (!data.isNull() && data.type() != QVariant::Map) {
-        *errorMessage = QCoreApplication::translate("ProjectExplorer::JsonWizard",
-                                                    "\"data\" must be empty or a JSON object for \"Project\" pages.");
+    if (!data.isNull() && data.typeId() != QMetaType::QVariantMap) {
+        *errorMessage = Tr::tr("\"data\" must be empty or a JSON object for \"Project\" pages.");
         return false;
     }
     QVariantMap tmp = data.toMap();
@@ -240,7 +252,7 @@ bool ProjectPageFactory::validateData(Core::Id typeId, const QVariant &data, QSt
     if (!projectNameValidator.isNull()) {
         QRegularExpression regularExpression(projectNameValidator);
         if (!regularExpression.isValid()) {
-            *errorMessage = QCoreApplication::translate("ProjectExplorer::JsonWizard",
+            *errorMessage = Tr::tr(
                 "Invalid regular expression \"%1\" in \"%2\". %3").arg(
                 projectNameValidator, QLatin1String(KEY_PROJECT_NAME_VALIDATOR), regularExpression.errorString());
             return false;
@@ -256,15 +268,24 @@ bool ProjectPageFactory::validateData(Core::Id typeId, const QVariant &data, QSt
 
 static const char KEY_HIDE_PROJECT_UI[] = "hideProjectUi";
 
+class SummaryPageFactory final : public JsonWizardPageFactory
+{
+public:
+    SummaryPageFactory();
+
+    WizardPage *create(JsonWizard *wizard, Id typeId, const QVariant &data) override;
+    bool validateData(Id typeId, const QVariant &data, QString *errorMessage) override;
+};
+
 SummaryPageFactory::SummaryPageFactory()
 {
     setTypeIdsSuffix(QLatin1String("Summary"));
 }
 
-Utils::WizardPage *SummaryPageFactory::create(JsonWizard *wizard, Core::Id typeId, const QVariant &data)
+WizardPage *SummaryPageFactory::create(JsonWizard *wizard, Id typeId, const QVariant &data)
 {
-    Q_UNUSED(wizard);
-    Q_UNUSED(data);
+    Q_UNUSED(wizard)
+    Q_UNUSED(data)
     QTC_ASSERT(canCreate(typeId), return nullptr);
 
     auto page = new JsonSummaryPage;
@@ -273,17 +294,26 @@ Utils::WizardPage *SummaryPageFactory::create(JsonWizard *wizard, Core::Id typeI
     return page;
 }
 
-bool SummaryPageFactory::validateData(Core::Id typeId, const QVariant &data, QString *errorMessage)
+bool SummaryPageFactory::validateData(Id typeId, const QVariant &data, QString *errorMessage)
 {
     QTC_ASSERT(canCreate(typeId), return false);
-    if (!data.isNull() && (data.type() != QVariant::Map)) {
-        *errorMessage = QCoreApplication::translate("ProjectExplorer::JsonWizard",
-            "\"data\" for a \"Summary\" page can be unset or needs to be an object.");
+    if (!data.isNull() && (data.typeId() != QMetaType::QVariantMap)) {
+        *errorMessage = Tr::tr("\"data\" for a \"Summary\" page can be unset or needs to be an object.");
         return false;
     }
 
     return true;
 }
 
-} // namespace Internal
-} // namespace ProjectExplorer
+// Setup
+
+void setupJsonWizardPages()
+{
+    static FieldPageFactory theFieldPageFactory;
+    static FilePageFactory theFilePageFactory;
+    static KitsPageFactory theKitsPageFactory;
+    static ProjectPageFactory theProjectPageFactory;
+    static SummaryPageFactory theSummaryPageFactory;
+}
+
+} // ProjectExplorer::Internal
