@@ -1,30 +1,10 @@
-/****************************************************************************
-**
-** Copyright (C) 2019 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2019 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "qmlpreviewplugin.h"
+
 #include "qmlpreviewruncontrol.h"
+#include "qmlpreviewtr.h"
 
 #ifdef WITH_TESTS
 #include "tests/qmlpreviewclient_test.h"
@@ -41,29 +21,38 @@
 #include <extensionsystem/pluginmanager.h>
 
 #include <projectexplorer/kit.h>
-#include <projectexplorer/kitinformation.h>
+#include <projectexplorer/kitaspects.h>
 #include <projectexplorer/project.h>
 #include <projectexplorer/projectexplorer.h>
 #include <projectexplorer/projectexplorerconstants.h>
+#include <projectexplorer/projectmanager.h>
 #include <projectexplorer/projectnodes.h>
 #include <projectexplorer/projecttree.h>
 #include <projectexplorer/runconfiguration.h>
-#include <projectexplorer/session.h>
 #include <projectexplorer/target.h>
 
 #include <qmljs/qmljsdocument.h>
 #include <qmljs/qmljsmodelmanagerinterface.h>
-#include <qmljstools/qmljstoolsconstants.h>
 
 #include <qmlprojectmanager/qmlmultilanguageaspect.h>
 
-#include <qtsupport/qtkitinformation.h>
+#include <qtsupport/qtkitaspect.h>
 #include <qtsupport/qtversionmanager.h>
 #include <qtsupport/baseqtversion.h>
 
+#include <texteditor/texteditor.h>
+
 #include <android/androidconstants.h>
 
+#include <utils/icon.h>
+#include <utils/mimeconstants.h>
+#include <utils/proxyaction.h>
+
 #include <QAction>
+#include <QMessageBox>
+#include <QPointer>
+#include <QTimer>
+#include <QToolBar>
 
 using namespace ProjectExplorer;
 
@@ -112,21 +101,21 @@ static std::unique_ptr<QmlDebugTranslationClient> defaultCreateDebugTranslationC
 {
     auto client = std::make_unique<QmlPreview::QmlDebugTranslationClient>(connection);
     return client;
-};
+}
 
+static void defaultRefreshTranslationFunction()
+{}
 
 class QmlPreviewPluginPrivate : public QObject
 {
 public:
     explicit QmlPreviewPluginPrivate(QmlPreviewPlugin *parent);
 
-    void previewCurrentFile();
     void onEditorChanged(Core::IEditor *editor);
     void onEditorAboutToClose(Core::IEditor *editor);
     void setDirty();
-    void addPreview(ProjectExplorer::RunControl *preview);
-    void removePreview(ProjectExplorer::RunControl *preview);
-    void attachToEditor();
+    void attachToEditorManager();
+    void detachFromEditorManager();
     void checkEditor();
     void checkFile(const QString &fileName);
     void triggerPreview(const QString &changedFile, const QByteArray &contents);
@@ -136,7 +125,7 @@ public:
     QmlPreviewRunControlList runningPreviews() const;
 
     QmlPreviewFileClassifier fileClassifier() const;
-    void setFileClassifier(QmlPreviewFileClassifier fileClassifer);
+    void setFileClassifier(QmlPreviewFileClassifier fileClassifier);
 
     float zoomFactor() const;
     void setZoomFactor(float zoomFactor);
@@ -150,102 +139,91 @@ public:
     QmlPreviewPlugin *q = nullptr;
     QThread m_parseThread;
     QString m_previewedFile;
-    QmlPreviewFileLoader m_fileLoader = nullptr;
-    Core::IEditor *m_lastEditor = nullptr;
+    QPointer<Core::IEditor> m_lastEditor;
     QmlPreviewRunControlList m_runningPreviews;
     bool m_dirty = false;
-    QmlPreview::QmlPreviewFileClassifier m_fileClassifer = nullptr;
-    float m_zoomFactor = -1.0;
-    QmlPreview::QmlPreviewFpsHandler m_fpsHandler = nullptr;
     QString m_localeIsoCode;
-    QmlDebugTranslationClientCreator m_createDebugTranslationClientMethod;
 
-    RunWorkerFactory localRunWorkerFactory{
-        RunWorkerFactory::make<LocalQmlPreviewSupport>(),
-        {Constants::QML_PREVIEW_RUN_MODE},
-        {}, // All runconfig.
-        {Constants::DESKTOP_DEVICE_TYPE}
-    };
+    LocalQmlPreviewSupportFactory localRunWorkerFactory;
 
-    RunWorkerFactory runWorkerFactory{
-        [this](RunControl *runControl) {
-            QmlPreviewRunner *runner = new QmlPreviewRunner(QmlPreviewRunnerSetting{
-                runControl,
-                m_fileLoader,
-                m_fileClassifer,
-                m_fpsHandler,
-                m_zoomFactor,
-                m_localeIsoCode,
-                m_createDebugTranslationClientMethod
-            });
-            connect(q, &QmlPreviewPlugin::updatePreviews,
-                    runner, &QmlPreviewRunner::loadFile);
-            connect(q, &QmlPreviewPlugin::rerunPreviews,
-                    runner, &QmlPreviewRunner::rerun);
-            connect(runner, &QmlPreviewRunner::ready,
-                    this, &QmlPreviewPluginPrivate::previewCurrentFile);
-            connect(q, &QmlPreviewPlugin::zoomFactorChanged,
-                    runner, &QmlPreviewRunner::zoom);
-            connect(q, &QmlPreviewPlugin::localeIsoCodeChanged,
-                    runner, &QmlPreviewRunner::language);
-
-            connect(runner, &RunWorker::started, this, [this, runControl] {
-                addPreview(runControl);
-            });
-            connect(runner, &RunWorker::stopped, this, [this, runControl] {
-                removePreview(runControl);
-            });
-
-            return runner;
-        },
-        {Constants::QML_PREVIEW_RUNNER}
-    };
+    QmlPreviewRunnerSetting m_settings;
+    QmlPreviewRunWorkerFactory runWorkerFactory;
 };
 
 QmlPreviewPluginPrivate::QmlPreviewPluginPrivate(QmlPreviewPlugin *parent)
-    : q(parent)
+    : q(parent), runWorkerFactory(parent, &m_settings)
 {
-    m_fileLoader = &defaultFileLoader;
-    m_fileClassifer = &defaultFileClassifier;
-    m_fpsHandler = &defaultFpsHandler;
-    m_createDebugTranslationClientMethod = &defaultCreateDebugTranslationClientMethod;
+    m_settings.fileLoader = &defaultFileLoader;
+    m_settings.fileClassifier = &defaultFileClassifier;
+    m_settings.fpsHandler = &defaultFpsHandler;
+    m_settings.createDebugTranslationClientMethod = &defaultCreateDebugTranslationClientMethod;
+    m_settings.refreshTranslationsFunction = &defaultRefreshTranslationFunction;
 
     Core::ActionContainer *menu = Core::ActionManager::actionContainer(
                 Constants::M_BUILDPROJECT);
-    QAction *action = new QAction(QmlPreviewPlugin::tr("QML Preview"), this);
-    action->setToolTip(QLatin1String("Preview changes to QML code live in your application."));
-    action->setEnabled(SessionManager::startupProject() != nullptr);
-    connect(SessionManager::instance(), &SessionManager::startupProjectChanged, action,
+    QAction *runPreviewAction = new QAction(Tr::tr("QML Preview"), this);
+    runPreviewAction->setToolTip(Tr::tr("Preview changes to QML code live in your application."));
+    runPreviewAction->setEnabled(ProjectManager::startupProject() != nullptr);
+    connect(ProjectManager::instance(), &ProjectManager::startupProjectChanged, runPreviewAction,
             &QAction::setEnabled);
-    connect(action, &QAction::triggered, this, [this]() {
+    connect(runPreviewAction, &QAction::triggered, this, [&, runPreviewAction] {
+        runPreviewAction->setEnabled(false);
         if (auto multiLanguageAspect = QmlProjectManager::QmlMultiLanguageAspect::current())
             m_localeIsoCode = multiLanguageAspect->currentLocale();
         bool skipDeploy = false;
-        const Kit *kit = SessionManager::startupTarget()->kit();
-        if (SessionManager::startupTarget() && kit)
+        const Kit *kit = ProjectManager::startupTarget()->kit();
+        if (ProjectManager::startupTarget() && kit)
             skipDeploy = kit->supportedPlatforms().contains(Android::Constants::ANDROID_DEVICE_TYPE)
                 || DeviceTypeKitAspect::deviceTypeId(kit) == Android::Constants::ANDROID_DEVICE_TYPE;
         ProjectExplorerPlugin::runStartupProject(Constants::QML_PREVIEW_RUN_MODE, skipDeploy);
     });
     menu->addAction(
-        Core::ActionManager::registerAction(action, "QmlPreview.RunPreview"),
+        Core::ActionManager::registerAction(runPreviewAction, "QmlPreview.RunPreview"),
         Constants::G_BUILD_RUN);
 
     menu = Core::ActionManager::actionContainer(Constants::M_FILECONTEXT);
-    action = new QAction(QmlPreviewPlugin::tr("Preview File"), this);
-    action->setEnabled(false);
-    connect(q, &QmlPreviewPlugin::runningPreviewsChanged,
-            action, [action](const QmlPreviewRunControlList &previews) {
-        action->setEnabled(!previews.isEmpty());
-    });
-    connect(action, &QAction::triggered, this, &QmlPreviewPluginPrivate::previewCurrentFile);
+    QAction *previewFileAction = new QAction(Tr::tr("Preview File"), this);
+    connect(previewFileAction, &QAction::triggered, q, &QmlPreviewPlugin::previewCurrentFile);
     menu->addAction(
-        Core::ActionManager::registerAction(action, "QmlPreview.PreviewFile",  Core::Context(Constants::C_PROJECT_TREE)),
+        Core::ActionManager::registerAction(previewFileAction, "QmlPreview.PreviewFile",
+                                            Core::Context(Constants::C_PROJECT_TREE)),
         Constants::G_FILE_OTHER);
-    action->setVisible(false);
-    connect(ProjectTree::instance(), &ProjectTree::currentNodeChanged, action, [action](Node *node) {
-        const FileNode *fileNode = node ? node->asFileNode() : nullptr;
-        action->setVisible(fileNode ? fileNode->fileType() == FileType::QML : false);
+    previewFileAction->setVisible(false);
+    connect(ProjectTree::instance(), &ProjectTree::currentNodeChanged, previewFileAction,
+            [previewFileAction] (Node *node) {
+                const FileNode *fileNode = node ? node->asFileNode() : nullptr;
+                previewFileAction->setVisible(fileNode && fileNode->fileType() == FileType::QML);
+            });
+    connect(Core::EditorManager::instance(), &Core::EditorManager::editorOpened, this,
+            [] (Core::IEditor *editor) {
+        if (!editor)
+            return;
+        if (!editor->document())
+            return;
+
+        if (const QString mimeType = editor->document()->mimeType();
+            mimeType != Utils::Constants::QML_MIMETYPE
+            && mimeType != Utils::Constants::QMLUI_MIMETYPE) {
+            return;
+        }
+
+        auto *textEditor = qobject_cast<TextEditor::BaseTextEditor *>(editor);
+        if (!textEditor)
+            return;
+        TextEditor::TextEditorWidget *widget = textEditor->editorWidget();
+        if (!widget)
+            return;
+        QToolBar *toolBar = widget->toolBar();
+        if (!toolBar)
+            return;
+
+        const QIcon icon = Utils::Icon({
+            {":/utils/images/run_small.png", Utils::Theme::IconsRunToolBarColor},
+            {":/utils/images/eyeoverlay.png", Utils::Theme::IconsDebugColor}
+        }).icon();
+        Utils::ProxyAction *action = Utils::ProxyAction::proxyActionWithIcon(
+                    Core::ActionManager::command("QmlPreview.RunPreview")->action(), icon);
+        toolBar->insertAction(nullptr, action);
     });
 
     m_parseThread.start();
@@ -255,8 +233,6 @@ QmlPreviewPluginPrivate::QmlPreviewPluginPrivate(QmlPreviewPlugin *parent)
     connect(q, &QmlPreviewPlugin::checkDocument, parser, &QmlPreviewParser::parse);
     connect(q, &QmlPreviewPlugin::previewedFileChanged, this, &QmlPreviewPluginPrivate::checkFile);
     connect(parser, &QmlPreviewParser::success, this, &QmlPreviewPluginPrivate::triggerPreview);
-
-    attachToEditor();
 }
 
 QmlPreviewPlugin::~QmlPreviewPlugin()
@@ -264,14 +240,14 @@ QmlPreviewPlugin::~QmlPreviewPlugin()
     delete d;
 }
 
-bool QmlPreviewPlugin::initialize(const QStringList &arguments, QString *errorString)
+void QmlPreviewPlugin::initialize()
 {
-    Q_UNUSED(arguments)
-    Q_UNUSED(errorString)
-
     d = new QmlPreviewPluginPrivate(this);
 
-    return true;
+#ifdef WITH_TESTS
+    addTestCreator(createQmlPreviewClientTest);
+    addTestCreator(createQmlPreviewPluginTest);
+#endif
 }
 
 ExtensionSystem::IPlugin::ShutdownFlag QmlPreviewPlugin::aboutToShutdown()
@@ -281,16 +257,6 @@ ExtensionSystem::IPlugin::ShutdownFlag QmlPreviewPlugin::aboutToShutdown()
     return SynchronousShutdown;
 }
 
-QVector<QObject *> QmlPreviewPlugin::createTestObjects() const
-{
-    QVector<QObject *> tests;
-#ifdef WITH_TESTS
-    tests.append(new QmlPreviewClientTest);
-    tests.append(new QmlPreviewPluginTest);
-#endif
-    return tests;
-}
-
 QString QmlPreviewPlugin::previewedFile() const
 {
     return d->m_previewedFile;
@@ -310,51 +276,57 @@ QmlPreviewRunControlList QmlPreviewPlugin::runningPreviews() const
     return d->m_runningPreviews;
 }
 
+void QmlPreviewPlugin::stopAllPreviews()
+{
+    for (auto &runningPreview : d->m_runningPreviews)
+        runningPreview->initiateStop();
+}
+
 QmlPreviewFileLoader QmlPreviewPlugin::fileLoader() const
 {
-    return d->m_fileLoader;
+    return d->m_settings.fileLoader;
 }
 
 QmlPreviewFileClassifier QmlPreviewPlugin::fileClassifier() const
 {
-    return d->m_fileClassifer;
+    return d->m_settings.fileClassifier;
 }
 
-void QmlPreviewPlugin::setFileClassifier(QmlPreviewFileClassifier fileClassifer)
+void QmlPreviewPlugin::setFileClassifier(QmlPreviewFileClassifier fileClassifier)
 {
-    if (d->m_fileClassifer == fileClassifer)
+    if (d->m_settings.fileClassifier == fileClassifier)
         return;
 
-    d->m_fileClassifer = fileClassifer;
-    emit fileClassifierChanged(d->m_fileClassifer);
+    d->m_settings.fileClassifier = fileClassifier;
+    emit fileClassifierChanged(d->m_settings.fileClassifier);
 }
 
 float QmlPreviewPlugin::zoomFactor() const
 {
-    return d->m_zoomFactor;
+    return d->m_settings.zoomFactor;
 }
 
 void QmlPreviewPlugin::setZoomFactor(float zoomFactor)
 {
-    if (d->m_zoomFactor == zoomFactor)
+    if (d->m_settings.zoomFactor == zoomFactor)
         return;
 
-    d->m_zoomFactor = zoomFactor;
-    emit zoomFactorChanged(d->m_zoomFactor);
+    d->m_settings.zoomFactor = zoomFactor;
+    emit zoomFactorChanged(d->m_settings.zoomFactor);
 }
 
 QmlPreviewFpsHandler QmlPreviewPlugin::fpsHandler() const
 {
-    return d->m_fpsHandler;
+    return d->m_settings.fpsHandler;
 }
 
 void QmlPreviewPlugin::setFpsHandler(QmlPreviewFpsHandler fpsHandler)
 {
-    if (d->m_fpsHandler == fpsHandler)
+    if (d->m_settings.fpsHandler == fpsHandler)
         return;
 
-    d->m_fpsHandler = fpsHandler;
-    emit fpsHandlerChanged(d->m_fpsHandler);
+    d->m_settings.fpsHandler = fpsHandler;
+    emit fpsHandlerChanged(d->m_settings.fpsHandler);
 }
 
 QString QmlPreviewPlugin::localeIsoCode() const
@@ -373,39 +345,49 @@ void QmlPreviewPlugin::setLocaleIsoCode(const QString &localeIsoCode)
     emit localeIsoCodeChanged(d->m_localeIsoCode);
 }
 
-void QmlPreviewPlugin::setQmlDebugTranslationClientCreator(QmlDebugTranslationClientCreator creator)
+void QmlPreviewPlugin::setQmlDebugTranslationClientCreator(QmlDebugTranslationClientFactoryFunction creator)
 {
-    d->m_createDebugTranslationClientMethod = creator;
+    d->m_settings.createDebugTranslationClientMethod = creator;
+}
+
+void QmlPreviewPlugin::setRefreshTranslationsFunction(QmlPreviewRefreshTranslationFunction refreshTranslationsFunction)
+{
+    d->m_settings.refreshTranslationsFunction = refreshTranslationsFunction;
 }
 
 void QmlPreviewPlugin::setFileLoader(QmlPreviewFileLoader fileLoader)
 {
-    if (d->m_fileLoader == fileLoader)
+    if (d->m_settings.fileLoader == fileLoader)
         return;
 
-    d->m_fileLoader = fileLoader;
-    emit fileLoaderChanged(d->m_fileLoader);
+    d->m_settings.fileLoader = fileLoader;
+    emit fileLoaderChanged(d->m_settings.fileLoader);
 }
 
-void QmlPreviewPluginPrivate::previewCurrentFile()
+void QmlPreviewPlugin::previewCurrentFile()
 {
     const Node *currentNode = ProjectTree::currentNode();
     if (!currentNode || !currentNode->asFileNode()
             || currentNode->asFileNode()->fileType() != FileType::QML)
         return;
 
+    if (runningPreviews().isEmpty())
+        QMessageBox::warning(Core::ICore::dialogParent(), Tr::tr("QML Preview Not Running"),
+                             Tr::tr("Start the QML Preview for the project before selecting "
+                                    "a specific file for preview."));
+
     const QString file = currentNode->filePath().toString();
-    if (file != m_previewedFile)
-        q->setPreviewedFile(file);
+    if (file != d->m_previewedFile)
+        setPreviewedFile(file);
     else
-        checkFile(file);
+        d->checkFile(file);
 }
 
 void QmlPreviewPluginPrivate::onEditorChanged(Core::IEditor *editor)
 {
-    if (m_lastEditor) {
-        Core::IDocument *doc = m_lastEditor->document();
-        disconnect(doc, &Core::IDocument::contentsChanged, this, &QmlPreviewPluginPrivate::setDirty);
+    if (m_lastEditor && m_lastEditor->document()) {
+        disconnect(m_lastEditor->document(), &Core::IDocument::contentsChanged,
+                   this, &QmlPreviewPluginPrivate::setDirty);
         if (m_dirty) {
             m_dirty = false;
             checkEditor();
@@ -439,7 +421,7 @@ void QmlPreviewPluginPrivate::onEditorAboutToClose(Core::IEditor *editor)
 void QmlPreviewPluginPrivate::setDirty()
 {
     m_dirty = true;
-    QTimer::singleShot(1000, this, [this](){
+    QTimer::singleShot(1000, this, [&](){
         if (m_dirty && m_lastEditor) {
             m_dirty = false;
             checkEditor();
@@ -447,19 +429,28 @@ void QmlPreviewPluginPrivate::setDirty()
     });
 }
 
-void QmlPreviewPluginPrivate::addPreview(ProjectExplorer::RunControl *preview)
+void QmlPreviewPlugin::addPreview(RunControl *preview)
 {
-    m_runningPreviews.append(preview);
-    emit q->runningPreviewsChanged(m_runningPreviews);
+    d->attachToEditorManager();
+    d->setDirty();
+    d->onEditorChanged(Core::EditorManager::currentEditor());
+
+    d->m_runningPreviews.append(preview);
+    emit runningPreviewsChanged(d->m_runningPreviews);
 }
 
-void QmlPreviewPluginPrivate::removePreview(ProjectExplorer::RunControl *preview)
+void QmlPreviewPlugin::removePreview(RunControl *preview)
 {
-    m_runningPreviews.removeOne(preview);
-    emit q->runningPreviewsChanged(m_runningPreviews);
+    d->m_runningPreviews.removeOne(preview);
+    emit runningPreviewsChanged(d->m_runningPreviews);
+    if (d->m_runningPreviews.isEmpty()) {
+        if (auto cmd = Core::ActionManager::command("QmlPreview.RunPreview"); cmd && cmd->action())
+            cmd->action()->setEnabled(true);
+        d->detachFromEditorManager();
+    }
 }
 
-void QmlPreviewPluginPrivate::attachToEditor()
+void QmlPreviewPluginPrivate::attachToEditorManager()
 {
     Core::EditorManager *editorManager = Core::EditorManager::instance();
     connect(editorManager, &Core::EditorManager::currentEditorChanged,
@@ -468,29 +459,39 @@ void QmlPreviewPluginPrivate::attachToEditor()
             this, &QmlPreviewPluginPrivate::onEditorAboutToClose);
 }
 
+void QmlPreviewPluginPrivate::detachFromEditorManager()
+{
+    Core::EditorManager *editorManager = Core::EditorManager::instance();
+    disconnect(editorManager, &Core::EditorManager::currentEditorChanged,
+               this, &QmlPreviewPluginPrivate::onEditorChanged);
+    disconnect(editorManager, &Core::EditorManager::editorAboutToClose,
+               this, &QmlPreviewPluginPrivate::onEditorAboutToClose);
+}
+
 void QmlPreviewPluginPrivate::checkEditor()
 {
     QmlJS::Dialect::Enum dialect = QmlJS::Dialect::AnyLanguage;
     Core::IDocument *doc = m_lastEditor->document();
+    using namespace Utils::Constants;
     const QString mimeType = doc->mimeType();
-    if (mimeType == QmlJSTools::Constants::JS_MIMETYPE)
+    if (mimeType == JS_MIMETYPE)
         dialect = QmlJS::Dialect::JavaScript;
-    else if (mimeType == QmlJSTools::Constants::JSON_MIMETYPE)
+    else if (mimeType == JSON_MIMETYPE)
         dialect = QmlJS::Dialect::Json;
-    else if (mimeType == QmlJSTools::Constants::QML_MIMETYPE)
+    else if (mimeType == QML_MIMETYPE)
         dialect = QmlJS::Dialect::Qml;
 //  --- Can we detect those via mime types?
 //  else if (mimeType == ???)
 //      dialect = QmlJS::Dialect::QmlQtQuick1;
 //  else if (mimeType == ???)
 //      dialect = QmlJS::Dialect::QmlQtQuick2;
-    else if (mimeType == QmlJSTools::Constants::QBS_MIMETYPE)
+    else if (mimeType == QBS_MIMETYPE)
         dialect = QmlJS::Dialect::QmlQbs;
-    else if (mimeType == QmlJSTools::Constants::QMLPROJECT_MIMETYPE)
+    else if (mimeType == QMLPROJECT_MIMETYPE)
         dialect = QmlJS::Dialect::QmlProject;
-    else if (mimeType == QmlJSTools::Constants::QMLTYPES_MIMETYPE)
+    else if (mimeType == QMLTYPES_MIMETYPE)
         dialect = QmlJS::Dialect::QmlTypeInfo;
-    else if (mimeType == QmlJSTools::Constants::QMLUI_MIMETYPE)
+    else if (mimeType == QMLUI_MIMETYPE)
         dialect = QmlJS::Dialect::QmlQtQuick2Ui;
     else
         dialect = QmlJS::Dialect::NoLanguage;
@@ -499,22 +500,25 @@ void QmlPreviewPluginPrivate::checkEditor()
 
 void QmlPreviewPluginPrivate::checkFile(const QString &fileName)
 {
-    if (!m_fileLoader)
+    if (!m_settings.fileLoader)
         return;
 
     bool success = false;
-    const QByteArray contents = m_fileLoader(fileName, &success);
+    const QByteArray contents = m_settings.fileLoader(fileName, &success);
 
     if (success) {
-        emit q->checkDocument(fileName, contents,
-                              QmlJS::ModelManagerInterface::guessLanguageOfFile(fileName).dialect());
+        emit q->checkDocument(fileName,
+                              contents,
+                              QmlJS::ModelManagerInterface::guessLanguageOfFile(
+                                  Utils::FilePath::fromUserInput(fileName))
+                                  .dialect());
     }
 }
 
 void QmlPreviewPluginPrivate::triggerPreview(const QString &changedFile, const QByteArray &contents)
 {
     if (m_previewedFile.isEmpty())
-        previewCurrentFile();
+        q->previewCurrentFile();
     else
         emit q->updatePreviews(m_previewedFile, changedFile, contents);
 }
@@ -533,7 +537,8 @@ void QmlPreviewParser::parse(const QString &name, const QByteArray &contents,
         return;
     }
 
-    QmlJS::Document::MutablePtr qmljsDoc = QmlJS::Document::create(name, dialect);
+    QmlJS::Document::MutablePtr qmljsDoc = QmlJS::Document::create(Utils::FilePath::fromString(name),
+                                                                   dialect);
     qmljsDoc->setSource(QString::fromUtf8(contents));
     if (qmljsDoc->parse())
         emit success(name, contents);
