@@ -1,43 +1,19 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
-#include "transitionitem.h"
 #include "connectableitem.h"
 #include "cornergrabberitem.h"
-#include "finalstateitem.h"
 #include "graphicsitemprovider.h"
-#include "graphicsscene.h"
-#include "parallelitem.h"
 #include "sceneutils.h"
 #include "scxmldocument.h"
 #include "scxmleditorconstants.h"
+#include "scxmleditortr.h"
 #include "scxmltagutils.h"
 #include "scxmluifactory.h"
 #include "serializer.h"
 #include "stateitem.h"
 #include "tagtextitem.h"
+#include "transitionitem.h"
 
 #include <QBrush>
 #include <QDebug>
@@ -52,6 +28,26 @@ using namespace ScxmlEditor::PluginInterface;
 
 const qreal SELECTION_DISTANCE = 10;
 
+static QString wrapText(const QString &text)
+{
+    QString wrappedText = "[" + text.trimmed() + "]";
+    return wrappedText;
+
+}
+
+static QString unwrapText(const QString &text)
+{
+    QString unwrappedText = text.trimmed();
+
+    if (unwrappedText.startsWith("["))
+        unwrappedText = unwrappedText.mid(1);
+
+    if (unwrappedText.endsWith("]"))
+        unwrappedText.chop(1);
+
+    return unwrappedText;
+}
+
 TransitionItem::TransitionItem(BaseItem *parent)
     : BaseItem(parent)
     , m_startTargetFactor(0.5, 0.5)
@@ -71,12 +67,21 @@ TransitionItem::TransitionItem(BaseItem *parent)
             << QPointF(0, 1);
 
     m_eventTagItem = new TagTextItem(this);
-    connect(m_eventTagItem, &TagTextItem::selected, this, [=](bool sel){
+    connect(m_eventTagItem, &TagTextItem::selected, this, [this](bool sel) {
         setItemSelected(sel);
     });
     connect(m_eventTagItem, &TagTextItem::textReady, this, &TransitionItem::textHasChanged);
     connect(m_eventTagItem, &TagTextItem::movePointChanged, this, &TransitionItem::textItemPositionChanged);
 
+    m_condTagItem = new TagTextItem(this);
+    connect(m_condTagItem, &TagTextItem::selected, this, [this](bool sel) { setItemSelected(sel); });
+    connect(m_condTagItem, &TagTextItem::textReady, this, [this](const QString &text) {
+        setTagValue("cond", unwrapText(text));
+        updateEventName();
+    });
+    connect(m_condTagItem, &TagTextItem::movePointChanged,
+            this, &TransitionItem::textItemPositionChanged);
+
     checkWarningItems();
 }
 
@@ -104,8 +109,10 @@ void TransitionItem::setTag(ScxmlTag *tag)
 {
     BaseItem::setTag(tag);
     if (tag) {
-        if (tag->tagType() == InitialTransition)
+        if (tag->tagType() == InitialTransition) {
             m_eventTagItem->setVisible(false);
+            m_condTagItem->setVisible(false);
+        }
     }
 }
 
@@ -120,12 +127,22 @@ void TransitionItem::textItemPositionChanged()
     }
     setEditorInfo(Constants::C_SCXML_EDITORINFO_MOVEPOINT, data);
 
+    QPointF p2 = m_condTagItem->movePoint();
+    QString data2;
+    if (p2.toPoint() != QPoint(0, 0)) {
+        Serializer s;
+        s.append(p2);
+        data2 = s.data();
+    }
+    setEditorInfo(Constants::C_SCXML_EDITORINFO_MOVEPOINT_COND, data2);
+
     updateComponents();
 }
 
 void TransitionItem::textHasChanged(const QString &text)
 {
     setTagValue("event", text);
+    updateComponents();
 }
 
 void TransitionItem::createGrabbers()
@@ -280,7 +297,8 @@ void TransitionItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
         if (bLeftButton) {
             // If we found QuickTransition-item or CornerGrabber at this point, we must ignore mouse press here
             // So we can press QuickTransition/CornerGrabber item although there is transition lines front of these items
-            foreach (QGraphicsItem *item, scene()->items(event->scenePos())) {
+            const QList<QGraphicsItem *> items = scene()->items(event->scenePos());
+            for (QGraphicsItem *item : items) {
                 if (item->type() == QuickTransitionType || (item->type() == CornerGrabberType && item->parentItem() != this)) {
                     event->ignore();
                     return;
@@ -306,19 +324,11 @@ void TransitionItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
                     QPointF intersPoint;
                     QLineF line2(p, p + QPointF(SELECTION_DISTANCE, SELECTION_DISTANCE));
                     line2.setAngle(line.angle() + 90);
-#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
-                    if (line.intersect(line2, &intersPoint) == QLineF::BoundedIntersection)
-#else
-                    if (line.intersects(line2, &intersPoint) == QLineF::BoundedIntersection)
-#endif
+                    if (line.intersects(line2, &intersPoint) == QLineF::BoundedIntersection) {
                         sel = true;
-                    else {
+                    } else {
                         line2.setAngle(line.angle() - 90);
-#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
-                        sel = line.intersect(line2, &intersPoint) == QLineF::BoundedIntersection;
-#else
                         sel = line.intersects(line2, &intersPoint) == QLineF::BoundedIntersection;
-#endif
                     }
 
                     if (sel)
@@ -424,7 +434,7 @@ void TransitionItem::createContextMenu(QMenu *menu)
     if (m_selectedGrabberIndex > 0) {
         data[Constants::C_SCXMLTAG_ACTIONTYPE] = TagUtils::RemovePoint;
         data["cornerIndex"] = m_selectedGrabberIndex;
-        menu->addAction(tr("Remove Point"))->setData(data);
+        menu->addAction(Tr::tr("Remove Point"))->setData(data);
     }
 
     menu->addSeparator();
@@ -457,7 +467,7 @@ void TransitionItem::selectedMenuAction(const QAction *action)
 
 void TransitionItem::keyPressEvent(QKeyEvent *event)
 {
-    if (event->key() == Qt::Key_Delete) {
+    if (event->key() == Qt::Key_Delete || event->key() == Qt::Key_Backspace) {
         bool bFound = false;
         if (m_cornerGrabbers.count() > 2) {
             for (int i = m_cornerGrabbers.count() - 1; i >= 1; i--) {
@@ -708,7 +718,7 @@ void TransitionItem::setEndPos(const QPointF &endPos, bool snap)
     }
 }
 
-void TransitionItem::setEndItem(ConnectableItem *item)
+void TransitionItem::setEndItem(ConnectableItem *item, bool fixValue)
 {
 
     if (item) {
@@ -725,7 +735,7 @@ void TransitionItem::setEndItem(ConnectableItem *item)
     }
 
     updateZValue();
-    updateTarget();
+    updateTarget(fixValue);
 }
 
 QPointF TransitionItem::loadPoint(const QString &name)
@@ -805,11 +815,7 @@ QPointF TransitionItem::findIntersectionPoint(ConnectableItem *item, const QLine
         for (int i = 1; i < itemPolygon.count(); ++i) {
             p2 = itemPolygon.at(i) + item->scenePos();
             checkLine = QLineF(p1, p2);
-#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
-            if (checkLine.intersect(line, &intersectPoint) == QLineF::BoundedIntersection)
-#else
             if (checkLine.intersects(line, &intersectPoint) == QLineF::BoundedIntersection)
-#endif
                 return intersectPoint;
             p1 = p2;
         }
@@ -911,13 +917,18 @@ void TransitionItem::updateComponents()
     QLineF nameLine(m_cornerPoints[ind], m_cornerPoints[ind + 1]);
     if (m_targetType <= InternalNoTarget) {
         m_eventTagItem->setPos(m_cornerPoints[1].x() + 6, m_cornerPoints[1].y() - m_eventTagItem->boundingRect().height() / 3);
+        m_condTagItem->setPos(m_cornerPoints[1].x() + 6, m_cornerPoints[1].y() + m_condTagItem->boundingRect().height() / 3);
     } else {
-        const qreal w2 = m_eventTagItem->boundingRect().width() / 2;
-        QPointF startPos = nameLine.pointAt(0.5);
-        QLineF targetLine(startPos, startPos + QPointF(SELECTION_DISTANCE, SELECTION_DISTANCE));
-        targetLine.setAngle(nameLine.angle() + 90);
+        const qreal widthEventItem = m_eventTagItem->boundingRect().width() / 2;
+        m_eventTagItem->setPos(
+            nameLine.pointAt(0.5)
+            + QPointF(-widthEventItem, -m_eventTagItem->boundingRect().height() / 1.5));
+
+        const qreal width = qMax(nameLine.length(), m_eventTagItem->boundingRect().width());
+        m_condTagItem->setTextMaxWidth(width);
+        const qreal widthCondItem = m_condTagItem->boundingRect().width() / 2;
+        m_condTagItem->setPos(nameLine.pointAt(0.5) + QPointF(-widthCondItem, 2));
 
-        m_eventTagItem->setPos(targetLine.p2() + m_eventTagItem->movePoint() - QPointF(w2, m_eventTagItem->boundingRect().height() / 2));
     }
 
     if (m_warningItem)
@@ -979,14 +990,16 @@ void TransitionItem::updateEditorInfo(bool allChilds)
 
     const QColor fontColor = editorInfo(Constants::C_SCXML_EDITORINFO_FONTCOLOR);
     m_eventTagItem->setDefaultTextColor(fontColor.isValid() ? fontColor : Qt::black);
+    m_condTagItem->setDefaultTextColor(fontColor.isValid() ? fontColor : Qt::black);
 
     const QColor stateColor = editorInfo(Constants::C_SCXML_EDITORINFO_STATECOLOR);
     m_pen.setColor(stateColor.isValid() ? stateColor : qRgb(0x12, 0x12, 0x12));
 }
 
-void TransitionItem::updateTarget()
+void TransitionItem::updateTarget(bool fixValue)
 {
-    setTagValue("target", m_endItem ? m_endItem->itemId() : QString());
+    if (fixValue)
+        setTagValue("target", m_endItem ? m_endItem->itemId() : QString());
     if (m_endItem)
         m_endItem->checkInitial(true);
 }
@@ -1002,7 +1015,7 @@ void TransitionItem::updateAttributes()
 
         m_endItem = nullptr;
         findEndItem();
-        updateTarget();
+        updateTarget(false);
         updateZValue();
     }
 
@@ -1060,6 +1073,7 @@ void TransitionItem::readUISpecifiedProperties(const ScxmlTag *tag)
             }
 
             m_eventTagItem->resetMovePoint(loadPoint(Constants::C_SCXML_EDITORINFO_MOVEPOINT));
+            m_condTagItem->resetMovePoint(loadPoint(Constants::C_SCXML_EDITORINFO_MOVEPOINT_COND));
 
             if (m_lineSelected)
                 createGrabbers();
@@ -1082,6 +1096,7 @@ void TransitionItem::finalizeCreation()
 void TransitionItem::checkVisibility(double scaleFactor)
 {
     m_eventTagItem->setVisible(scaleFactor > 0.5);
+    m_condTagItem->setVisible(scaleFactor > 0.5);
 }
 
 bool TransitionItem::containsScenePoint(const QPointF &p) const
@@ -1095,19 +1110,11 @@ bool TransitionItem::containsScenePoint(const QPointF &p) const
         QPointF intersPoint;
         QLineF line2(pp, pp + QPointF(SELECTION_DISTANCE, SELECTION_DISTANCE));
         line2.setAngle(line.angle() + 90);
-#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
-        if (line.intersect(line2, &intersPoint) == QLineF::BoundedIntersection) {
-#else
         if (line.intersects(line2, &intersPoint) == QLineF::BoundedIntersection) {
-#endif
             return true;
         } else {
             line2.setAngle(line.angle() - 90);
-#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
-            if (line.intersect(line2, &intersPoint) == QLineF::BoundedIntersection)
-#else
             if (line.intersects(line2, &intersPoint) == QLineF::BoundedIntersection)
-#endif
                 return true;
         }
     }
@@ -1123,7 +1130,7 @@ void TransitionItem::findEndItem()
             if (items[i]->type() >= FinalStateType) {
                 auto item = qgraphicsitem_cast<ConnectableItem*>(items[i]);
                 if (item && item->itemId() == targetId) {
-                    setEndItem(item);
+                    setEndItem(item, false);
                     break;
                 }
             }
@@ -1134,6 +1141,8 @@ void TransitionItem::findEndItem()
 void TransitionItem::updateEventName()
 {
     m_eventTagItem->setText(tagValue("event"));
+    m_condTagItem->setText(wrapText(tagValue("cond")));
+    updateComponents();
 }
 
 void TransitionItem::storeGeometry(bool block)
@@ -1165,6 +1174,11 @@ void TransitionItem::storeMovePoint(bool block)
         setEditorInfo(Constants::C_SCXML_EDITORINFO_MOVEPOINT, QString(), block);
     else
         savePoint(m_eventTagItem->movePoint(), Constants::C_SCXML_EDITORINFO_MOVEPOINT);
+
+    if (m_condTagItem->movePoint().toPoint() == QPoint(0, 0))
+        setEditorInfo(Constants::C_SCXML_EDITORINFO_MOVEPOINT_COND, QString(), block);
+    else
+        savePoint(m_condTagItem->movePoint(), Constants::C_SCXML_EDITORINFO_MOVEPOINT_COND);
 }
 
 void TransitionItem::storeTargetFactors(bool block)
@@ -1218,6 +1232,7 @@ void TransitionItem::updateTargetType()
 
     if (type <= InternalNoTarget) {
         m_eventTagItem->resetMovePoint();
+        m_condTagItem->resetMovePoint();
         m_arrowSize = 6;
         // Remove extra points
         while (m_cornerPoints.count() > 2)
@@ -1229,8 +1244,10 @@ void TransitionItem::updateTargetType()
         setEditorInfo(Constants::C_SCXML_EDITORINFO_GEOMETRY, QString(), true);
         setEditorInfo(Constants::C_SCXML_EDITORINFO_LOCALGEOMETRY, QString(), true);
         setEditorInfo(Constants::C_SCXML_EDITORINFO_MOVEPOINT, QString(), true);
+        setEditorInfo(Constants::C_SCXML_EDITORINFO_MOVEPOINT_COND, QString(), true);
         setEditorInfo(Constants::C_SCXML_EDITORINFO_STARTTARGETFACTORS, QString(), true);
         setEditorInfo(Constants::C_SCXML_EDITORINFO_ENDTARGETFACTORS, QString(), true);
+        updateComponents();
     } else {
         m_arrowSize = 10;
     }
@@ -1286,10 +1303,10 @@ void TransitionItem::updateZValue()
 
 qreal TransitionItem::textWidth() const
 {
-    return m_eventTagItem->boundingRect().width();
+    return m_eventTagItem->boundingRect().width() + m_condTagItem->boundingRect().width();
 }
 
 QRectF TransitionItem::wholeBoundingRect() const
 {
-    return boundingRect().united(m_eventTagItem->sceneBoundingRect());
+    return boundingRect().united(m_eventTagItem->sceneBoundingRect().united(m_condTagItem->sceneBoundingRect()));
 }
