@@ -1,27 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include <QtTest>
 #include <QObject>
@@ -29,8 +7,8 @@
 #include <QTextDocument>
 #include <QTextBlock>
 
-#include <cpptools/cppcodeformatter.h>
-using namespace CppTools;
+#include <cppeditor/cppcodeformatter.h>
+using namespace CppEditor;
 
 class tst_CodeFormatter: public QObject
 {
@@ -128,6 +106,10 @@ private Q_SLOTS:
     void initializerWithinFunctionArg();
     void shiftWithinInitializer();
     void lambdaWithReturnType();
+    void structuredBinding();
+    void subscriptOperatorInFunctionCall();
+    void statementMacros();
+    void tryCatchClause();
 };
 
 struct Line {
@@ -173,23 +155,23 @@ struct Line {
     int expectedPadding;
 };
 
-QString concatLines(QList<Line> lines)
+QString concatLines(const QList<Line> &lines)
 {
     QString result;
-    foreach (const Line &l, lines) {
+    for (const Line &l : lines) {
         result += l.line;
         result += "\n";
     }
     return result;
 }
 
-void checkIndent(QList<Line> data, QtStyleCodeFormatter formatter)
+void checkIndent(const QList<Line> &data, QtStyleCodeFormatter formatter)
 {
     QString text = concatLines(data);
     QTextDocument document(text);
 
     int i = 0;
-    foreach (const Line &l, data) {
+    for (const Line &l : data) {
         QTextBlock b = document.findBlockByLineNumber(i);
         if (l.expectedIndent != -1) {
             int indent, padding;
@@ -204,14 +186,14 @@ void checkIndent(QList<Line> data, QtStyleCodeFormatter formatter)
     }
 }
 
-void checkIndent(QList<Line> data, CppCodeStyleSettings style)
+void checkIndent(QList<Line> &data, CppCodeStyleSettings style)
 {
     QtStyleCodeFormatter formatter;
     formatter.setCodeStyleSettings(style);
     checkIndent(data, formatter);
 }
 
-void checkIndent(QList<Line> data, int style = 0)
+void checkIndent(QList<Line> &data, int style = 0)
 {
     CppCodeStyleSettings codeStyle;
     QtStyleCodeFormatter formatter;
@@ -1565,8 +1547,8 @@ void tst_CodeFormatter::indentNamespace()
          << Line("};")
          << Line("}")
          << Line("int j;")
-         << Line("namespace {")
-         << Line("namespace Foo {")
+         << Line("inline namespace {")
+         << Line("inline namespace Foo {")
          << Line("int j;")
          << Line("}")
          << Line("int j;")
@@ -1606,9 +1588,9 @@ void tst_CodeFormatter::indentNamespace2()
          << Line("    };")
          << Line("}")
          << Line("int j;")
-         << Line("namespace {")
+         << Line("inline namespace {")
          << Line("    int j;")
-         << Line("    namespace Foo {")
+         << Line("    inline namespace Foo {")
          << Line("        int j;")
          << Line("    }")
          << Line("    int j;")
@@ -2209,7 +2191,87 @@ void tst_CodeFormatter::lambdaWithReturnType()
     checkIndent(data);
 }
 
-QTEST_MAIN(tst_CodeFormatter)
+void tst_CodeFormatter::structuredBinding()
+{
+    QList<Line> data;
+    data << Line("int main() {")
+         << Line("    auto [v1, v2] = tuple_func();")
+         << Line("    return 0;")
+         << Line("}")
+            ;
+    checkIndent(data);
+}
+
+void tst_CodeFormatter::subscriptOperatorInFunctionCall()
+{
+    QList<Line> data;
+    data << Line("int main() {")
+         << Line("    func(array[0],")
+         << Line("    ~    0);")
+         << Line("    func(array[i],")
+         << Line("    ~    i);")
+         << Line("}")
+        ;
+    checkIndent(data);
+}
+
+void tst_CodeFormatter::statementMacros()
+{
+    QList<Line> data;
+    data << Line("MY_MACRO")
+         << Line("template<int n = 0>")
+         << Line("~       class C;");
+    checkIndent(data);
+
+    data.clear();
+    CppCodeStyleSettings settings;
+    settings.statementMacros << "MY_MACRO";
+    data << Line("MY_MACRO")
+         << Line("template<int n = 0>")
+         << Line("class C;");
+    checkIndent(data, settings);
+}
+
+void tst_CodeFormatter::tryCatchClause()
+{
+    CppCodeStyleSettings settings;
+    settings.indentFunctionBody = true;
+    settings.indentFunctionBraces = false;
+    settings.indentBlockBody = false;
+    settings.indentBlockBraces = true;
+    QList<Line> data;
+    data << Line("int main()")
+         << Line("{")
+         << Line("    try")
+         << Line("        {")
+         << Line("        throw;")
+         << Line("        }")
+         << Line("    catch (const E &e)")
+         << Line("        {")
+         << Line("        handle(e);")
+         << Line("        }")
+         << Line("    catch (...)")
+         << Line("        {")
+         << Line("        handle();")
+         << Line("        }")
+         << Line("}");
+    checkIndent(data, settings);
+
+    data.clear();
+    data << Line("int main()")
+         << Line("{")
+         << Line("    try {")
+         << Line("        throw;")
+         << Line("    } catch (const E &e) {")
+         << Line("        handle(e);")
+         << Line("    } catch (...) {")
+         << Line("        handle();")
+         << Line("    }")
+         << Line("}");
+    checkIndent(data);
+}
+
+QTEST_GUILESS_MAIN(tst_CodeFormatter)
 
 #include "tst_codeformatter.moc"
 
